@@ -6,27 +6,8 @@
 #include <unity-misc/na-tray.h>
 
 static gboolean
-filter_tray (NaTray *tray, NaTrayChild *icon, gpointer null)
+on_window_expose (GtkWidget *window, cairo_t *cr)
 {
-  char *res_name = NULL;
-  char *res_class = NULL;
-
-  na_tray_child_get_wm_class (icon, &res_name, &res_class);
-  if (na_tray_child_has_alpha (icon))
-    na_tray_child_set_composited (icon, TRUE);
-
-  g_debug ("Child %s: %s %s", na_tray_child_get_title (icon), res_name, res_class);
-  
-  g_free (res_name);
-  g_free (res_class);
-
-  return TRUE;
-}
-
-static gboolean
-on_window_expose (GtkWidget *window, GdkEventExpose *event)
-{
-  cairo_t *cr = gdk_cairo_create (window->window);
   GtkAllocation alloc;
 
   gtk_widget_get_allocation (window, &alloc);
@@ -39,11 +20,9 @@ on_window_expose (GtkWidget *window, GdkEventExpose *event)
   cairo_rectangle (cr, 0, 0, alloc.width, alloc.height);
   cairo_fill (cr);
 
-  cairo_destroy (cr);
-
-  gtk_container_propagate_expose (GTK_CONTAINER (window),
+  gtk_container_propagate_draw (GTK_CONTAINER (window),
                                   gtk_bin_get_child (GTK_BIN (window)),
-                                  event);
+                                  cr);
 
   return FALSE;
 }
@@ -62,18 +41,16 @@ main (int argc, char *argv[])
   window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
   gtk_window_resize (GTK_WINDOW (window), 100, 24);
   gtk_widget_set_name (window, "UnityPanelApplet");
-  gtk_widget_set_colormap (window, gdk_screen_get_rgba_colormap (gdk_screen_get_default ()));
+  gtk_widget_set_visual (window, gdk_screen_get_rgba_visual (gdk_screen_get_default ()));
   gtk_widget_realize (window);
-  gdk_window_set_back_pixmap (window->window, NULL, FALSE);
   gtk_widget_set_app_paintable (window, TRUE);
 
-  g_signal_connect (window, "expose-event",
+  g_signal_connect (window, "draw",
                     G_CALLBACK (on_window_expose), NULL);
 
   tray = (GtkWidget *)na_tray_new_for_screen (gdk_screen_get_default (),
                                               GTK_ORIENTATION_HORIZONTAL,
-                                              (NaTrayFilterCallback)filter_tray,
-                                              NULL);
+                                              NULL, NULL);
   gtk_container_add (GTK_CONTAINER (window), tray);
   gtk_widget_show (tray);
 
